@@ -3,21 +3,22 @@
  *
  *	General database support functions.
  *
- *	Copyright (c) 2003-2006, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: dbutil.c,v 1.7.2.1 2006-01-06 17:07:48 cbbrowne Exp $
+ *	$Id: dbutil.c,v 1.12.2.1 2007-04-18 19:28:27 cbbrowne Exp $
  *-------------------------------------------------------------------------
  */
 
 
+#ifndef WIN32
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-
+#endif
 #include "postgres.h"
 #include "libpq-fe.h"
 
@@ -69,7 +70,6 @@ db_notice_recv(void *arg, const PGresult *res)
 				PQresultErrorMessage(res));
 	}
 }
-
 #else							/* !HAVE_PQSETNOTICERECEIVER */
 
 /* ----------
@@ -111,6 +111,7 @@ int
 db_connect(SlonikStmt * stmt, SlonikAdmInfo * adminfo)
 {
 	PGconn	   *dbconn;
+	SlonDString	query;
 
 	db_notice_stmt = stmt;
 
@@ -136,7 +137,18 @@ db_connect(SlonikStmt * stmt, SlonikAdmInfo * adminfo)
 #else
 	PQsetNoticeProcessor(dbconn, db_notice_recv, NULL);
 #endif   /* !HAVE_PQSETNOTICERECEIVER */
+
+	dstring_init(&query);
+	slon_mkquery(&query,"SET datestyle to 'ISO'");
+
 	adminfo->dbconn = dbconn;
+	if (db_exec_command(stmt, adminfo, &query) < 0)
+	{
+		printf("Unable to set datestyle\n");
+		dstring_free(&query);
+		return -1;
+	}
+	dstring_free(&query); 
 	return 0;
 }
 
@@ -314,10 +326,15 @@ db_get_nodeid(SlonikStmt * stmt, SlonikAdmInfo * adminfo)
  * ----------
  */
 int
-db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo, int *major, int *minor)
+db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo)
 {
 	PGresult   *res;
 	SlonDString query;
+	char        versionstr[7];
+	int	    major=0;
+	int	    minor=0;
+	int         patch=0;
+	int         version=0;
 
 	if (db_begin_xact(stmt, adminfo) < 0)
 		return -1;
@@ -330,7 +347,7 @@ db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo, int *major, int *mino
 	if (res == NULL)
 		return -1;
 
-	if (sscanf(PQgetvalue(res, 0, 0), "PostgreSQL %d.%d", major, minor) != 2)
+	if (sscanf(PQgetvalue(res, 0, 0), "PostgreSQL %d.%d.%d", &major, &minor, &patch) < 2)
 	{
 		fprintf(stderr, "%s:%d: failed to parse %s for DB version\n",
 				stmt->stmt_filename, stmt->stmt_lno,
@@ -339,8 +356,9 @@ db_get_version(SlonikStmt * stmt, SlonikAdmInfo * adminfo, int *major, int *mino
 		return -1;
 	}
 	PQclear(res);
-
-	return 0;
+	snprintf(versionstr, 7, "%.2d%.2d%.2d", major, minor, patch);
+	version=atoi(versionstr);
+	return version;
 }
 
 
@@ -601,7 +619,7 @@ static int
 slon_appendquery_int(SlonDString * dsp, char *fmt, va_list ap)
 {
 	char	   *s;
-	char		buf    [64];
+	char		buf[64];
 
 	while (*fmt)
 	{
@@ -619,7 +637,7 @@ slon_appendquery_int(SlonDString * dsp, char *fmt, va_list ap)
 
 					case 'q':
 						s = va_arg(ap, char *);
-						while (*s != '\0')
+						while (s && *s != '\0')
 						{
 							switch (*s)
 							{
