@@ -6,27 +6,26 @@
  *	Copyright (c) 2003-2004, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slonik.c,v 1.67.2.3 2006/12/12 14:52:59 xfade Exp $
+ *	$Id: slonik.c,v 1.67.2.11 2007-06-07 19:36:14 wieck Exp $
  *-------------------------------------------------------------------------
  */
 
 
+#ifndef WIN32
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#ifndef WIN32
 #include <unistd.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <errno.h>
+#include <time.h>
 #else
 #define sleep(x) Sleep(x*1000)
 #define vsnprintf _vsnprintf
-#define INT64_FORMAT "%I64d"
 #endif
-#include <errno.h>
-#include <time.h>
 
 #include "postgres.h"
 #include "libpq-fe.h"
@@ -1882,17 +1881,22 @@ load_slony_base(SlonikStmt * stmt, int no_id)
 		use_major = 7;
 		use_minor = 4;
 	}
-	else if ((adminfo->pg_version >= 70500) && adminfo->pg_version < 80300)	/* 8.0, 8.1 and 8.2 */ 
+	else if ((adminfo->pg_version >= 70500) && adminfo->pg_version < 80100)	/* 8.0 */
 	{
 		use_major = 8;
 		use_minor = 0;
 	}
-	else	/* 8.3 and above */
+	else if ((adminfo->pg_version >= 80100) && adminfo->pg_version < 80400)	/* 8.1, 8.2 and 8.3 */
 	{
 		use_major = 8;
-		use_minor = 0;
+		use_minor = 1;
+	}
+	else	/* 8.3+ */
+	{
+		use_major = 8;
+		use_minor = 1;
 		printf("%s:%d: Possible unsupported PostgreSQL "
-			"version (%d) %d.%d, defaulting to 8.0 support\n",
+			"version (%d) %d.%d, defaulting to 8.1 support\n",
                         stmt->stmt_filename, stmt->stmt_lno, adminfo->pg_version,
 			(adminfo->pg_version/10000), ((adminfo->pg_version%10000)/100));
 	}
@@ -1977,17 +1981,22 @@ load_slony_functions(SlonikStmt * stmt, int no_id)
                 use_major = 7;
                 use_minor = 4;
         }
-        else if ((adminfo->pg_version >= 70500) && adminfo->pg_version < 80200) /* 8.0 and 8.1 */
+        else if ((adminfo->pg_version >= 70500) && adminfo->pg_version < 80100) /* 8.0 and 8.1 */
         {
                 use_major = 8;
                 use_minor = 0;
         }
-        else    /* 8.2 and above */
+       else if ((adminfo->pg_version >= 80100) && adminfo->pg_version < 80300)	/* 8.1 and 8.2 */
+       {
+	       use_major = 8;
+	       use_minor = 1;
+       }
+        else    /* 8.3 and above */
         {
                 use_major = 8;
-                use_minor = 0;
+                use_minor = 1;
                 printf("%s:%d: Possible unsupported PostgreSQL "
-                        "version %d.%d, defaulting to 8.0 support\n",
+                        "version %d.%d, defaulting to 8.1 support\n",
                         stmt->stmt_filename, stmt->stmt_lno,
                         (adminfo->pg_version/10000), ((adminfo->pg_version%10000)/100));
         }
@@ -3034,11 +3043,13 @@ slonik_uninstall_node(SlonikStmt_uninstall_node * stmt)
 				 stmt->hdr.script->clustername);
 	if (db_exec_command((SlonikStmt *) stmt, adminfo1, &query) < 0)
 	{
+		printf("Failed to exec uninstallNode() for node %d\n", stmt->no_id);
 		dstring_free(&query);
 		return -1;
 	}
 	if (db_commit_xact((SlonikStmt *) stmt, adminfo1) < 0)
 	{
+		printf("Failed to commit uninstallNode() for node %d\n", stmt->no_id);
 		dstring_free(&query);
 		return -1;
 	}
@@ -3048,11 +3059,13 @@ slonik_uninstall_node(SlonikStmt_uninstall_node * stmt)
 				 stmt->hdr.script->clustername);
 	if (db_exec_command((SlonikStmt *) stmt, adminfo1, &query) < 0)
 	{
+		printf("Failed to drop schema for node %d\n", stmt->no_id);
 		dstring_free(&query);
 		return -1;
 	}
 	if (db_commit_xact((SlonikStmt *) stmt, adminfo1) < 0)
 	{
+		printf("Failed to commit drop schema for node %d\n", stmt->no_id);
 		dstring_free(&query);
 		return -1;
 	}
@@ -3835,13 +3848,18 @@ slonik_ddl_script(SlonikStmt_ddl_script * stmt)
 	PGresult *res;
 	ExecStatusType rstat;
 
+
 #define PARMCOUNT 1  
 
         const char *params[PARMCOUNT];
         int paramlens[PARMCOUNT];
         int paramfmts[PARMCOUNT];
 
-	adminfo1 = get_active_adminfo((SlonikStmt *) stmt, stmt->ev_origin);
+	if(stmt->only_on_node > -1) {
+		adminfo1 = get_active_adminfo((SlonikStmt*) stmt,stmt->only_on_node);
+	} else {
+		adminfo1 = get_active_adminfo((SlonikStmt *) stmt, stmt->ev_origin);
+	}
 	if (adminfo1 == NULL)
 		return -1;
 
@@ -3868,6 +3886,7 @@ slonik_ddl_script(SlonikStmt_ddl_script * stmt)
 	slon_mkquery(&query,
 		     "select \"_%s\".ddlScript_prepare(%d, %d); ",
 		     stmt->hdr.script->clustername,
+
 		     stmt->ddl_setid, /* dstring_data(&script),  */ 
 		     stmt->only_on_node);
 
@@ -3915,39 +3934,59 @@ slonik_ddl_script(SlonikStmt_ddl_script * stmt)
 			rstat = PQresultStatus(res);
 			printf("DDL Statement failed - %s\n", PQresStatus(rstat));
 			dstring_free(&query);
+			PQclear(res);
 			return -1;
 		}
 		/* rstat = PQresultStatus(res); */
 		/* printf ("Success - %s\n", PQresStatus(rstat)); */
+
+		PQclear(res);
 	}
 	
-	printf("Submit DDL Event to subscribers...\n");
-
+	printf("Complete DDL Event...\n");
+	
 	slon_mkquery(&query, "select \"_%s\".ddlScript_complete(%d, $1::text, %d); ", 
-		     stmt->hdr.script->clustername,
-		     stmt->ddl_setid,  
-		     stmt->only_on_node);
-
+				 stmt->hdr.script->clustername,
+				 stmt->ddl_setid,  
+				 stmt->only_on_node);
+	
 	paramlens[PARMCOUNT-1] = 0;
 	paramfmts[PARMCOUNT-1] = 0;
 	params[PARMCOUNT-1] = dstring_data(&script);
-
-	res = PQexecParams(adminfo1->dbconn, dstring_data(&query), PARMCOUNT,
-			   NULL, params, paramlens, paramfmts, 0);
 	
-	if (PQresultStatus(res) != PGRES_COMMAND_OK && 
-	    PQresultStatus(res) != PGRES_TUPLES_OK &&
-	    PQresultStatus(res) != PGRES_EMPTY_QUERY)
+	res = PQexecParams(adminfo1->dbconn, dstring_data(&query), PARMCOUNT,
+					   NULL, params, paramlens, paramfmts, 0);
+	
+	/*
+	 * Check that the event submission returned exactly one result row
+	 */
+	if (PQresultStatus(res) != PGRES_TUPLES_OK)
 	{
 		rstat = PQresultStatus(res);
 		printf("Event submission for DDL failed - %s\n", PQresStatus(rstat));
+		dstring_free(&script);
 		dstring_free(&query);
+		PQclear(res);
 		return -1;
-	} else {
-		rstat = PQresultStatus(res);
-		printf ("DDL on origin - %s\n", PQresStatus(rstat));
 	}
-	
+	if (PQntuples(res) != 1)
+	{
+		printf("Event submission for DDL failed - query returned %d rows (expected 1)\n", PQntuples(res));
+		dstring_free(&script);
+		dstring_free(&query);
+		PQclear(res);
+		return -1;
+	}
+
+	/*
+	 * ... which should be the event-seqno for the DDL_SCRIPT event.
+	 * We remember that in the admin conninfo for WAIT.
+	 */
+	slon_scanint64(PQgetvalue(res, 0, 0), &(adminfo1->last_event));
+
+	rstat = PQresultStatus(res);
+	PQclear(res);
+	printf ("DDL submission to initial node - %s\n", PQresStatus(rstat));
 	dstring_free(&script);
 	dstring_free(&query);
 	return 0;
@@ -4216,7 +4255,7 @@ slonik_sync(SlonikStmt_sync * stmt)
 				 "select \"_%s\".createEvent('_%s', 'SYNC'); ",
 				 stmt->hdr.script->clustername,
 				 stmt->hdr.script->clustername);
-	if (db_exec_command((SlonikStmt *) stmt, adminfo1, &query) < 0)
+	if (db_exec_evcommand((SlonikStmt *) stmt, adminfo1, &query) < 0)
 	{
 		dstring_free(&query);
 		return -1;
